@@ -60,6 +60,7 @@ module Text.XML.Stream.Parse
     , def
     , DecodeEntities
     , psDecodeEntities
+    , psRetainNamespaces
       -- *** Entity decoding
     , decodeXmlEntities
     , decodeHtmlEntities
@@ -93,6 +94,7 @@ import Data.Attoparsec.Text
     , manyTill, takeWhile, try, anyChar
     )
 import qualified Control.Applicative as A
+import Control.Monad.Trans.Resource (MonadThrow, monadThrow, MonadResource)
 import Data.Conduit.Attoparsec (conduitParser, PositionRange)
 import Data.XML.Types
     ( Name (..), Event (..), Content (..)
@@ -118,7 +120,7 @@ import Data.Conduit
 import qualified Data.Conduit.Text as CT
 import qualified Data.Conduit.List as CL
 import qualified Data.Conduit.Internal as CI
-import Control.Monad (ap, liftM)
+import Control.Monad (ap, liftM, void, guard)
 import qualified Data.Text as TS
 import Data.List (foldl')
 import Data.Typeable (Typeable)
@@ -127,28 +129,44 @@ import Data.Conduit.Binary (sourceFile)
 import Data.Char (isSpace)
 import Data.Default (Default (..))
 import Control.Monad.Trans.Class (lift)
+import Data.Maybe (fromMaybe, isNothing)
 
 type Ents = [(Text, Text)]
 
-tokenToEvent :: Ents -> [NSLevel] -> Token -> (Ents, [NSLevel], [Event])
-tokenToEvent es n (TokenBeginDocument _) = (es, n, [])
-tokenToEvent es n (TokenInstruction i) = (es, n, [EventInstruction i])
-tokenToEvent es n (TokenBeginElement name as isClosed _) =
+tokenToEvent :: ParseSettings -> Ents -> [NSLevel] -> Token -> (Ents, [NSLevel], [Event])
+tokenToEvent _ es n (TokenBeginDocument _) = (es, n, [])
+tokenToEvent _ es n (TokenInstruction i) = (es, n, [EventInstruction i])
+tokenToEvent ps es n (TokenBeginElement name as isClosed _) =
     (es, n', if isClosed then [begin, end] else [begin])
   where
     l0 = case n of
             [] -> NSLevel Nothing Map.empty
             x:_ -> x
     (as', l') = foldl' go (id, l0) as
-    go (front, l) (TName kpref kname, val)
-        | kpref == Just "xmlns" =
-            (front, l { prefixes = Map.insert kname (contentsToText val)
-                                 $ prefixes l })
-        | kpref == Nothing && kname == "xmlns" =
-            (front, l { defaultNS = if T.null $ contentsToText val
-                                        then Nothing
-                                        else Just $ contentsToText val })
-        | otherwise = (front . (:) (TName kpref kname, map resolve val), l)
+    go (front, l) (TName kpref kname, val) =
+        (addNS front, l'')
+      where
+        isPrefixed = kpref == Just "xmlns"
+        isUnprefixed = isNothing kpref && kname == "xmlns"
+
+        addNS
+            | not (psRetainNamespaces ps) && (isPrefixed || isUnprefixed) = id
+            | otherwise = (((tname, map resolve val):) .)
+          where
+            tname
+                | isPrefixed = TName Nothing ("xmlns:" `T.append` kname)
+                | otherwise = TName kpref kname
+
+        l''
+            | isPrefixed =
+                l { prefixes = Map.insert kname (contentsToText val)
+                                     $ prefixes l }
+            | isUnprefixed =
+                l { defaultNS = if T.null $ contentsToText val
+                                            then Nothing
+                                            else Just $ contentsToText val }
+            | otherwise = l
+
     resolve (ContentEntity e)
         | Just t <- lookup e es = ContentText t
     resolve c = c
@@ -157,19 +175,19 @@ tokenToEvent es n (TokenBeginElement name as isClosed _) =
     elementName = tnameToName False l' name
     begin = EventBeginElement elementName $ map fixAttName $ as' []
     end = EventEndElement elementName
-tokenToEvent es n (TokenEndElement name) =
+tokenToEvent _ es n (TokenEndElement name) =
     (es, n', [EventEndElement $ tnameToName False l name])
   where
     (l, n') =
         case n of
             [] -> (NSLevel Nothing Map.empty, [])
             x:xs -> (x, xs)
-tokenToEvent es n (TokenContent (ContentEntity e))
+tokenToEvent _ es n (TokenContent (ContentEntity e))
     | Just t <- lookup e es = (es, n, [EventContent $ ContentText t])
-tokenToEvent es n (TokenContent c) = (es, n, [EventContent c])
-tokenToEvent es n (TokenComment c) = (es, n, [EventComment c])
-tokenToEvent es n (TokenDoctype t eid es') = (es ++ es', n, [EventBeginDoctype t eid, EventEndDoctype])
-tokenToEvent es n (TokenCDATA t) = (es, n, [EventCDATA t])
+tokenToEvent _ es n (TokenContent c) = (es, n, [EventContent c])
+tokenToEvent _ es n (TokenComment c) = (es, n, [EventComment c])
+tokenToEvent _ es n (TokenDoctype t eid es') = (es ++ es', n, [EventBeginDoctype t eid, EventEndDoctype])
+tokenToEvent _ es n (TokenCDATA t) = (es, n, [EventCDATA t])
 
 tnameToName :: Bool -> NSLevel -> TName -> Name
 tnameToName _ _ (TName (Just "xml") name) =
@@ -263,7 +281,7 @@ parseText :: MonadThrow m
 parseText de =
     dropBOM
         =$= tokenize
-        =$= toEventC
+        =$= toEventC de
         =$= addBeginEnd
   where
     tokenize = conduitToken de
@@ -272,8 +290,8 @@ parseText de =
         (yield (Nothing, EventEndDocument))
         (\e -> yield e >> addEnd)
 
-toEventC :: Monad m => Conduit (PositionRange, Token) m EventPos
-toEventC =
+toEventC :: Monad m => ParseSettings -> Conduit (PositionRange, Token) m EventPos
+toEventC ps =
     go [] []
   where
     go !es !levels =
@@ -282,15 +300,25 @@ toEventC =
         push (position, token) =
             mapM_ (yield . (,) (Just position)) events >> go es' levels'
           where
-            (es', levels', events) = tokenToEvent es levels token
+            (es', levels', events) = tokenToEvent ps es levels token
 
 data ParseSettings = ParseSettings
     { psDecodeEntities :: DecodeEntities
+    , psRetainNamespaces :: Bool
+    -- ^ Whether the original xmlns attributes should be retained in the parsed
+    -- values. For more information on motivation, see:
+    --
+    -- <https://github.com/snoyberg/xml/issues/38>
+    --
+    -- Default: False
+    --
+    -- Since 1.2.1
     }
 
 instance Default ParseSettings where
     def = ParseSettings
         { psDecodeEntities = decodeXmlEntities
+        , psRetainNamespaces = False
         }
 
 conduitToken :: MonadThrow m => ParseSettings -> Conduit TS.Text m (PositionRange, Token)
@@ -469,7 +497,7 @@ newline :: Parser ()
 newline = ((char '\r' >> char '\n') <|> char '\n') >> return ()
 
 char' :: Char -> Parser ()
-char' c = char c >> return ()
+char' = void . char
 
 data ContentType =
     Ignore | IsContent Text | IsError String | NotContent
@@ -510,11 +538,7 @@ contentMaybe = do
 -- | Grabs the next piece of content. If none if available, returns 'T.empty'.
 -- This is simply a wrapper around 'contentMaybe'.
 content :: MonadThrow m => Consumer Event m Text
-content = do
-    x <- contentMaybe
-    case x of
-        Nothing -> return T.empty
-        Just y -> return y
+content = fromMaybe T.empty <$> contentMaybe
 
 -- | The most generic way to parse a tag. It takes a predicate for checking if
 -- this is the correct tag name, an 'AttrParser' for handling attributes, and
@@ -526,9 +550,13 @@ content = do
 --
 -- This function automatically ignores comments, instructions and whitespace.
 tag :: MonadThrow m
-    => (Name -> Maybe a)
-    -> (a -> AttrParser b)
-    -> (b -> CI.ConduitM Event o m c)
+    => (Name -> Maybe a) -- ^ Check if this is a correct tag name
+                         --   and return a value that can be used to get an @AttrParser@.
+                         --   If this returns @Nothing@, the function will also return @Nothing@
+    -> (a -> AttrParser b) -- ^ Given the value returned by the name checker, this function will
+                           --   be used to get an @AttrParser@ appropriate for the specific tag.
+    -> (b -> CI.ConduitM Event o m c) -- ^ Handler function to handle the attributes and children
+                                      --   of a tag, given the value return from the @AttrParser@
     -> CI.ConduitM Event o m (Maybe c)
 tag checkName attrParser f = do
     x <- dropWS
@@ -549,6 +577,7 @@ tag checkName attrParser f = do
                 Nothing -> return Nothing
         _ -> return Nothing
   where
+    -- Drop Events until we encount a non-whitespace element
     dropWS = do
         x <- CL.peek
         let isWS =
@@ -576,38 +605,45 @@ tag checkName attrParser f = do
 
 -- | A simplified version of 'tag' which matches against boolean predicates.
 tagPredicate :: MonadThrow m
-             => (Name -> Bool)
-             -> AttrParser a
-             -> (a -> CI.ConduitM Event o m b)
+             => (Name -> Bool) -- ^ Name predicate that returns @True@ if the name matches the parser
+             -> AttrParser a -- ^ The attribute parser to be used for tags matching the predicate
+             -> (a -> CI.ConduitM Event o m b) -- ^ Handler function to handle the attributes and children
+                                               --   of a tag, given the value return from the @AttrParser@
              -> CI.ConduitM Event o m (Maybe b)
-tagPredicate p attrParser = tag (\x -> if p x then Just () else Nothing) (const attrParser)
+tagPredicate p attrParser = tag (guard . p) (const attrParser)
 
 -- | A simplified version of 'tag' which matches for specific tag names instead
 -- of taking a predicate function. This is often sufficient, and when combined
 -- with OverloadedStrings and the IsString instance of 'Name', can prove to be
 -- very concise.
+-- .
+-- Note that @Name@ is namespace sensitive. When using the @IsString@ instance of name,
+-- use
+-- > "{http://a/b}c" :: Name
+-- to match the tag @c@ in the XML namespace @http://a/b@
 tagName :: MonadThrow m
-     => Name
-     -> AttrParser a
-     -> (a -> CI.ConduitM Event o m b)
+     => Name -- ^ The tag name this parser matches to (includes namespaces)
+     -> AttrParser a -- ^ The attribute parser to be used for tags matching the predicate 
+     -> (a -> CI.ConduitM Event o m b) -- ^ Handler function to handle the attributes and children
+                                       --   of a tag, given the value return from the @AttrParser@
      -> CI.ConduitM Event o m (Maybe b)
 tagName name = tagPredicate (== name)
 
 -- | A further simplified tag parser, which requires that no attributes exist.
 tagNoAttr :: MonadThrow m
-          => Name
-          -> CI.ConduitM Event o m a
+          => Name -- ^ The name this parser matches to
+          -> CI.ConduitM Event o m a -- ^ Handler function to handle the children of the matched tag
           -> CI.ConduitM Event o m (Maybe a)
 tagNoAttr name f = tagName name (return ()) $ const f
 
 -- | Get the value of the first parser which returns 'Just'. If no parsers
--- succeed (i.e., return 'Just'), this function returns 'Nothing'.
+-- succeed (i.e., return @Just@), this function returns 'Nothing'.
 --
 -- > orE a b = choose [a, b]
 orE :: Monad m
-    => Consumer Event m (Maybe a)
-    -> Consumer Event m (Maybe a)
-    -> Consumer Event m (Maybe a)
+    => Consumer Event m (Maybe a) -- ^ The first (preferred) parser
+    -> Consumer Event m (Maybe a) -- ^ The second parser, only executed if the first parser fails
+    -> Consumer Event m (Maybe a) 
 orE a b = do
   x <- a
   case x of
@@ -617,8 +653,9 @@ orE a b = do
 -- | Get the value of the first parser which returns 'Just'. If no parsers
 -- succeed (i.e., return 'Just'), this function returns 'Nothing'.
 choose :: Monad m
-       => [Consumer Event m (Maybe a)]
-       -> Consumer Event m (Maybe a)
+       => [Consumer Event m (Maybe a)] -- ^ List of parsers that will be tried in order.
+       -> Consumer Event m (Maybe a) -- ^ Result of the first parser to succeed, or @Nothing@
+                                     --   if no parser succeeded
 choose [] = return Nothing
 choose (i:is) = do
     x <- i
@@ -631,7 +668,7 @@ choose (i:is) = do
 -- want to finally force something to happen.
 force :: MonadThrow m
       => String -- ^ Error message
-      -> CI.ConduitM Event o m (Maybe a)
+      -> CI.ConduitM Event o m (Maybe a) -- ^ Optional parser to be forced
       -> CI.ConduitM Event o m a
 force msg i = do
     x <- i
